/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.shotoku;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.log4j.Logger;
import org.apache.velocity.app.VelocityEngine;
import org.jboss.shotoku.exceptions.DeleteException;
import org.jboss.shotoku.exceptions.ResourceDoesNotExist;
import org.jboss.shotoku.exceptions.RepositoryException;
import org.jboss.shotoku.exceptions.SaveException;
import org.jboss.shotoku.search.Search;
import org.jboss.shotoku.tools.Constants;
import org.jboss.shotoku.tools.Pair;
import org.jboss.shotoku.tools.Tools;

/**
 * A base class which provides access to nodes and directories.
 * <br />
 * <b>Warning:</b> all shotoku classes are not thread-safe.
 * 
 * @author Adam Warski (adamw@aster.pl)
 * @author Damon Sicore (damon@sicore.com)
 */
public abstract class ContentManager {
	/**
	 * Prefix of this content manager.
	 */
	protected String prefix;

	/**
	 * Id of a corresponding repository.
	 */
	protected String id;

	/**
	 * A map of initialized velocity engines, The key
	 * is a string representation of parameters with which an engine was
	 * initialized.
	 */
	private static Map<String, VelocityEngine> engines;

	/**
	 * Gets a root directory that is represented by this content manager.
	 * 
	 * @return A directory that is the root directory of this content manager.
	 * @throws RepositoryException
	 */
	public abstract Directory getRootDirectory() throws RepositoryException;

	/**
	 * Gets the most recent version of a node which can be found under the given
	 * path (to get nodes from subdirectories, use <code>/</code> to separate 
	 * them).
	 * 
	 * @param path
	 *            Path for which to get the node.
	 * @return A node corresponding to the given path.
	 * @throws ResourceDoesNotExist
	 * @throws RepositoryException
	 */
	public abstract Node getNode(String path) throws ResourceDoesNotExist,
			RepositoryException;

	/**
	 * Gets a directory which can be found under the given path (to get nodes 
	 * from subdirectories, use <code>/</code> to separate them).
	 * 
	 * @param path
	 *            Path for which to get the directory.
	 * @return A directory corresponding to the gien path.
	 * @throws ResourceDoesNotExist
	 * @throws RepositoryException
	 */
	public abstract Directory getDirectory(String path)
			throws ResourceDoesNotExist, RepositoryException;

	/**
	 * Performs the given search in this content manager.
	 * 
	 * @param search
	 *            Search to perform.
	 * @return A node list that is the result of the search.
	 */
	public NodeList search(Search search) throws ResourceDoesNotExist {
		return search.perform(this);
	}

	/**
	 * Saves the given resources at one time. Recommended if you have multiple
	 * resources to save.
	 * 
	 * @param resources
	 *            Resources to save.
	 * @param logMessage
	 *            Log message to save with.
	 * @throws SaveException
	 * @throws RepositoryException
	 */
	public abstract void save(Collection<Resource> resources, String logMessage)
			throws SaveException, RepositoryException;

	/**
	 * Deletes the given resources at one time. Recommended if you have multiple
	 * resources to delete.
	 * 
	 * @param resources
	 *            Resources to delete.
	 * @throws DeleteException
	 * @throws RepositoryException
	 */
	public abstract void delete(Collection<Resource> resources)
			throws DeleteException, RepositoryException;
	
	/**
	 * Saves the given resources at one time. Equivalent to
	 * <code>save(Collection, String)</code>, where the collection is filled
	 * with the two given resources.
	 * 
	 * @param res1
	 *            First resource to save.
	 * @param res2
	 *            Second resource to save.
	 * @param logMessage
	 *            Log message to save with.
	 * @throws SaveException
	 * @throws RepositoryException
	 */
	public void save(Resource res1, Resource res2, String logMessage)
			throws SaveException, RepositoryException {
		List<Resource> toSave = new ArrayList<Resource>();
		toSave.add(res1);
		toSave.add(res2);
		save(toSave, logMessage);
	}

	/**
	 * Saves the given resources at one time. Equivalent to
	 * <code>save(Collection, String)</code>, where the collection is filled
	 * with the three given resources.
	 * 
	 * @param res1
	 *            First resource to save.
	 * @param res2
	 *            Second resource to save.
	 * @param res3
	 *            Third resource to save.
	 * @param logMessage
	 *            Log message to save with.
	 * @throws SaveException
	 * @throws RepositoryException
	 */
	public void save(Resource res1, Resource res2, Resource res3,
			String logMessage) throws SaveException, RepositoryException {
		List<Resource> toSave = new ArrayList<Resource>();
		toSave.add(res1);
		toSave.add(res2);
		toSave.add(res3);
		save(toSave, logMessage);
	}

	/**
	 * Saves the given resources at one time. Equivalent to
	 * <code>save(Collection, String)</code>, where the collection is filled
	 * with the four given resources.
	 * 
	 * @param res1
	 *            First resource to save.
	 * @param res2
	 *            Second resource to save.
	 * @param res3
	 *            Third resource to save.
	 * @param res4
	 *            Fourth resource to save.
	 * @param logMessage
	 *            Log message to save with.
	 * @throws SaveException
	 * @throws RepositoryException
	 */
	public void save(Resource res1, Resource res2, Resource res3,
			Resource res4, String logMessage) throws SaveException,
			RepositoryException {
		List<Resource> toSave = new ArrayList<Resource>();
		toSave.add(res1);
		toSave.add(res2);
		toSave.add(res3);
		toSave.add(res4);
		save(toSave, logMessage);
	}

	/**
	 * Saves the given resources at one time. Equivalent to
	 * <code>save(Collection, String)</code>, where the collection is filled
	 * with the five given resources.
	 * 
	 * @param res1
	 *            First resource to save.
	 * @param res2
	 *            Second resource to save.
	 * @param res3
	 *            Third resource to save.
	 * @param res4
	 *            Fourth resource to save.
	 * @param res5
	 *            Fifth resource to save.
	 * @param logMessage
	 *            Log message to save with.
	 * @throws SaveException
	 * @throws RepositoryException
	 */
	public void save(Resource res1, Resource res2, Resource res3,
			Resource res4, Resource res5, String logMessage)
			throws SaveException, RepositoryException {
		List<Resource> toSave = new ArrayList<Resource>();
		toSave.add(res1);
		toSave.add(res2);
		toSave.add(res3);
		toSave.add(res4);
		toSave.add(res5);
		save(toSave, logMessage);
	}

	/**
	 * Gets a velocity engine, initialized with default properties, as defined
	 * in velocity.properties in the shotoku-base jar. Additionaly, the shotoku
	 * resource loader will be set to read templates from this content manager
	 * (if shotoku resource loader is defined to be used).
	 * 
	 * @return A velocity engine initialized with default properties.
	 * @throws Exception
	 *             In case of a velocity exception/ properties reading
	 *             exception.
	 */
	public VelocityEngine getVelocityEngine() throws Exception {
		return getVelocityEngine(true);
	}

	/**
	 * Gets a velocity engine, initialized with default properties, as defined
	 * in velocity.properties in the shotoku-base jar.
	 * 
	 * @param addIdPrefix
	 *            True iff id/ prefix properties should be added to the
	 *            properties.
	 * @return A velocity engine initialized with default properties.
	 * @throws Exception
	 *             In case of a velocity exception/ properties reading
	 *             exception.
	 */
	public VelocityEngine getVelocityEngine(boolean addIdPrefix)
			throws Exception {
		return getVelocityEngine(new HashMap<Object, Object>(), addIdPrefix);
	}

	/**
	 * Gets a velocity engine, initialized with default properties, overwritten
	 * by the given propeties.
	 * 
	 * @param overProps
	 *            Properties which will be added to velocity configuration.
	 * @return An initialized velocity engine.
	 * @throws Exception
	 *             In case of a velocity exception/ properties reading
	 *             exception.
	 */
	public VelocityEngine getVelocityEngine(
			Map<? extends Object, ? extends Object> overProps,
			boolean addIdPrefix) throws Exception {
		String representation = "";
		for (Object key : overProps.keySet()) {
			representation += key + "=" + overProps.get(key) + ",";
		}

		representation += ";" + prefix + ";" + id + ";";
		representation += addIdPrefix;

		VelocityEngine ve = engines.get(representation);
		if (ve == null) {
			ve = new VelocityEngine();
			Properties props = new Properties();
			props.load(ContentManager.class
					.getResourceAsStream(Constants.VELOCITY_PROPERTIES_FILE));
			props.putAll(overProps);

			if (addIdPrefix) {
				props.put(Constants.VELOCITY_RL_PREFIX, prefix);
				props.put(Constants.VELOCITY_RL_ID, id);
			}

			ve.init(props);

			engines.put(representation, ve);
		}

		return ve;
	}

	/**
	 * Standard constructor.
	 * 
	 * @param id
	 *            Id of this content manager.
	 * @param prefix
	 *            Prefix of this content manager, as stated in properties.
	 * @throws RepositoryException
	 */
	protected ContentManager(String id, String prefix)
			throws RepositoryException {
		this.id = id;
		this.prefix = Tools.normalizeSlashes(prefix);
	}

	/*
	 * Content manager loading.
	 */

	private static final Logger log = Logger.getLogger(ContentManager.class);

	/**
	 * <code>defaultId</code> - id of the default content manager. May be
	 * overriden by a setting in the xml configuration file.
	 */
	private static String defaultId;

	/**
	 * <code>contentManagers</code> - map (content manager id -> content
	 * manager constructor).
	 */
	private static Map<String, Pair<Constructor, Configuration>> contentManagers;

	/**
	 * <code>props</code> - Shotoku properties, as read from the configuration
	 * file.
	 */
	private static PropertiesConfiguration props;

	/**
	 * <code>setupDone</code> - a marker to execute setup only once, even if
	 * the user does it many times by mistake.
	 */
	private static boolean setupDone;

	static {
		// Setup was not yet executed for sure.
		setupDone = false;

		// When this class is loaded, always reading configuration first.
		try {
			props = new PropertiesConfiguration();

			props.load(ContentManager.class
					.getResourceAsStream(Constants.PROPERTIES_FILE));
		} catch (Exception e) {
			log.warn("Error while loading content manager configuration: " + e);
		}
	}

	/**
	 * Gets a property of this content manager, as it is written in the
	 * configuration file (helper method for implementations).
	 * 
	 * @param name
	 *            Name of the property to get.
	 * @return Value of the given property or null, if no such property exists.
	 */
	public static String getProperty(String name) {
		return props.getString(name);
	}

	/**
	 * Setups the content managers before first used. In embedded mode, should
	 * be always called once, before any usage of Shotoku classes. In app server
	 * mode, should be never called by the user.
	 */
	public static void setup() {
		synchronized (ContentManager.class) {
			if (setupDone)
				return;

			// Preventing setup to be executed twice.
			setupDone = true;
		}

		engines = Collections
				.synchronizedMap(new HashMap<String, VelocityEngine>());

		contentManagers = new HashMap<String, Pair<Constructor, Configuration>>();

		try {
			// Reading the default id - possibly, overriding.
			defaultId = props.getString(Constants.PROPERTIES_ID_DEFAULT,
					Constants.DEFAULT_ID);

			// Getting ids of defined content managers.
			String[] ids = props.getStringArray(Constants.PROPERTIES_IDS);

			for (String id : ids) {
				Configuration conf = props.subset(Tools.concatenateProperties(
						Constants.PROPERTIES_PREFIX, id));

				String implementation = conf
						.getString(Constants.PROPERTIES_IMPL_SUFFIX);
				Class implClass = Class.forName(implementation);

				contentManagers.put(id, new Pair<Constructor, Configuration>(
						implClass.getConstructor(new Class[] { String.class,
								String.class, Configuration.class }), conf));

				// Executing the setup function.
				try {
					Method setupMethod = implClass.getMethod(
							Constants.SETUP_FUNCTION, new Class[] {
									String.class, Configuration.class });

					setupMethod.invoke(implClass, new Object[] { id, conf });
				} catch (NoSuchMethodException e) {
					// No setup - doing nothing.
				}

				log.info("Added content manager: " + id + ", "
								+ implementation);
			}
		} catch (Exception e) {
			e.printStackTrace();
			log.warn("Error setting up content manager: " + e);
		}
	}

	/**
	 * Gets a content manager with a default id and an empty prefix.
	 * 
	 * @return A ""-prefixed, default content manager, or null, if a default
	 *         content manager is not registered.
	 */
	public static ContentManager getContentManager() {
		return getContentManager("");
	}

	/**
	 * Gets a content manager with a default id and the given prefix.
	 * 
	 * @param prefix
	 *            Prefix for the new content manager (all paths will have this
	 *            prepended).
	 * @return A prefixed, default content manager, or null, if a default
	 *         content manager is not registered.
	 */
	public static ContentManager getContentManager(String prefix) {
		return getContentManager(defaultId, prefix);
	}

	/**
	 * Gets a content manager with the given id and prefix.
	 * 
	 * @param id
	 *            Id of the content manager.
	 * @param prefix
	 *            Prefix for the new content manager (all paths will have this
	 *            prepended).
	 * @return A prefixed content manager with the given id or null, if a
	 *         content manager with the given id registers.
	 */
	public static ContentManager getContentManager(String id, String prefix) {
		if (id == null)
			id = defaultId;
		if (prefix == null)
			prefix = "";
		else
			prefix = Tools.normalizeSlashes(prefix);

		Pair<Constructor, Configuration> cmData = contentManagers.get(id);

		if (cmData == null)
			return null;

		try {
			return (ContentManager) cmData.getFirst().newInstance(
					new Object[] { id, prefix, cmData.getSecond() });
		} catch (Throwable e) {
			log.warn("Unable to get a content manager: (" + id + ", " + prefix
					+ ").", e);
			return null;
		}
	}
}
