/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.shotoku.tools;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Utility, helper functions used internally.
 * 
 * @author Adam Warski (adamw@aster.pl)
 */
public class Tools {
	/**
	 * Reads text contained in a tag of the form <tag>text</tag>.
	 * 
	 * @param textNode
	 *            Node to read text from.
	 * @return Text contained in the given node.
	 */
	public static String unmarshallText(org.w3c.dom.Node textNode) {
		StringBuffer buf = new StringBuffer();

		org.w3c.dom.Node n;
		org.w3c.dom.NodeList nodes = textNode.getChildNodes();

		for (int i = 0; i < nodes.getLength(); i++) {
			n = nodes.item(i);

			if (n.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
				buf.append(n.getNodeValue());
			}
		}

		return buf.toString();
	}

	/**
	 * From the given node, gets the value of the specified attribute.
	 * 
	 * @param root
	 *            Node to read the attribute from.
	 * @param attrName
	 *            Name of the attribute to read.
	 * @return Value of the given attribute or null, if no such attribute is
	 *         present.
	 */
	public static String getAttributeValue(org.w3c.dom.Node root,
			String attrName) {
		org.w3c.dom.Node n = root.getAttributes().getNamedItem(attrName);

		if (n == null)
			return null;

		return unmarshallText(n);
	}
	
	
	/**
	 * From the given string, removes unnecessary / in the middle and
	 * ending. Also, ensures that there is a / at the beginning.
	 * For example: "///a//b//c/" is transformed to "/a/b/c".
	 * 
	 * @param toNormalize
	 *            String to normalize.
	 * @return <code>toNormalize</code> with unnecessary / removed.
	 */
	public static String normalizeSlashes(String toNormalize) {
		while (toNormalize.contains("//"))
			toNormalize = toNormalize.replace("//", "/");

		if (!toNormalize.startsWith("/"))
			toNormalize = "/" + toNormalize;

		if ((toNormalize.endsWith("/")) && (toNormalize.length() > 1))
			toNormalize = toNormalize.substring(0, toNormalize.length() - 1);

		return toNormalize;
	}
	
	/**
	 * Concatenates two parts of a path into 1: returns a string
	 * <code>path1/path2</code>.
	 * 
	 * @param path1
	 *            First path to concatenate.
	 * @param path2
	 *            Second path to concatenate.
	 * @return
	 */
	public static String concatenatePaths(String path1, String path2) {
		return path1 + "/" + path2;
	}
	
	/**
	 * Concatenates two parts of a property name: returns a string
	 * <code>property1.property2</code>.
	 * 
	 * @param property1
	 *            First property part to concatenate.
	 * @param property2
	 *            Second property part to concatenate.
	 * @return
	 */
	public static String concatenateProperties(String property1,
			String property2) {
		return property1 + "." + property2;
	}
	
	/**
	 * Transferes all bytes from the given input stream to the given output
	 * stream.
	 * 
	 * @param is
	 *            Input stream to read from.
	 * @param os
	 *            Outpu stream to write to.
	 * @throws IOException
	 */
	public static void transfer(InputStream is, OutputStream os) throws IOException {
		byte[] buffer = new byte[Constants.TRANSFER_BUF_SIZE];
		int read;
		while ((read = is.read(buffer)) != -1)
			os.write(buffer, 0, read);
	}
	
	/**
	 * Checks if the given string represents a true value. For example, "true",
	 * "yes", "TrUe" etc will return true.
	 * 
	 * @param s
	 *            String to check.
	 * @return True iff the given string can be determined to represent "true".
	 */
	public static boolean isTrue(String s) {
		if (s == null) {
			return false;
		}
		
		s = s.trim().toLowerCase();
		if ("true".equals(s) || "yes".equals(s)) {
			return true;
		}
		
		return false;
	}
}
