/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.shotoku.svn.service;

import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.log4j.Logger;
import org.jboss.shotoku.tools.Tools;
import org.tmatesoft.svn.core.SVNException;
import org.tmatesoft.svn.core.SVNURL;
import org.tmatesoft.svn.core.io.SVNRepository;
import org.tmatesoft.svn.core.io.SVNRepositoryFactory;
import org.tmatesoft.svn.core.wc.ISVNOptions;
import org.tmatesoft.svn.core.wc.SVNClientManager;
import org.tmatesoft.svn.core.wc.SVNRevision;
import org.tmatesoft.svn.core.wc.SVNUpdateClient;
import org.tmatesoft.svn.core.wc.SVNWCUtil;

/**
 * A class for performing operations on a single subversion repository and
 * its local working copy.
 * @author Adam Warski (adamw@aster.pl)
 * @author Damon Sicore (damon@sicore.com)
 */
public class SvnRepository {
	private static final Logger log = Logger.getLogger(SvnRepository.class);
	
	private SVNClientManager ourClientManager;
	private SVNRepository repository;
	private String url;
	private File wc;
	private Set<String> modifiedFiles;
	private Set<String> modifiedDirs;
	private Set<String> modifiedTrees;
	private Set<String> deletedResources;
	private Set<String> frozenModifiedFiles;
	private Set<String> frozenModifiedDirs;
	private Set<String> frozenModifiedTrees;
	private Set<String> frozenDeletedResources;
	
	/**
	 * <code>lastRevision</code> - last revision to which the wc was updated.
	 * Used to prevent unnecessary updates if the revision hasn't changed.
	 */
	private long lastRevision;
	
	private Object synchronizer;
	
	public SvnRepository(String username, String password,
			String url, String localpath) {
		ISVNOptions options = SVNWCUtil.createDefaultOptions(true);
    	ourClientManager = 
    		SVNClientManager.newInstance(options, username, password);
    	
    	try {
			repository = SVNRepositoryFactory.create(SVNURL.parseURIEncoded(
					url));
			repository.setAuthenticationManager(
					SVNWCUtil.createDefaultAuthenticationManager(username, 
							password));
		} catch (SVNException e) {
			log.warn("Error while creating SVNRepoistory.", e);
		}
    	
		lastRevision = -1;
		
    	this.url = url;

    	wc = new File(localpath);
    	
    	synchronizer = new Object();
    	
    	modifiedTrees = new HashSet<String>();
    	modifiedFiles = new HashSet<String>();
    	modifiedDirs = new HashSet<String>();
    	deletedResources = new HashSet<String>();
    	
    	frozenModifiedTrees = new HashSet<String>();
    	frozenModifiedFiles = new HashSet<String>();
    	frozenModifiedDirs = new HashSet<String>();
    	frozenDeletedResources = new HashSet<String>();
	}
	
	/**
	 * Tries to perform a cleanup on the local working copy. Called in case
	 * of exceptions that occure while performing other operations.
	 */
	private void tryCleanup() {
		try {
			ourClientManager.getWCClient().doCleanup(wc);
		} catch (Exception e) {

		}
	}
	
	/**
	 * Updates the local working copy, additionaly attempting a checkout if
	 * update fails.
	 */
	public void firstUpdate() {
		update(true, true);
	}
	
	/**
	 * Updates the local working copy.
	 */
	public void update() {
    	update(false, true);
    }
	
	/**
	 * Updates the repository. If it fails, and <code>tryCheckout</code> is
	 * set to true, a checkout is attempted; if it set to false, and
	 * <code>firstTime</code> is set to true, then we are trying to do a
	 * cleanup on the WC and execute this method once again, but this time with
	 * <code>firstTime</code> set to false, to prevent a loop.
	 * 
	 * @param tryCheckout
	 *            True iff a checkout is to be attempted on update failure.
	 * @param firstTime
	 *            True iff this is the first attempt to do an update.
	 */
	private synchronized void update(boolean tryCheckout, boolean firstTime) {
		/*
		 * Checking if there is any need for updating the working copy.
		 */
		try {
			if (lastRevision == repository.getLatestRevision()) {
				return;
			}
		} catch (SVNException e1) {
			// Trying to continue ...
		}
		
		/*
		 * Freezing all entries in the modified/ deleted sets. After update,
		 * these sets will be cleaned, because all changes that they stored
		 * information about will be reflected in the WC. But any files that
		 * are modified/ deleted during the update, have to be stored in 
		 * separate sets, and can't be removed from them after the update.
		 */
		synchronized (synchronizer) {
			frozenModifiedFiles = modifiedFiles;
			modifiedFiles = new HashSet<String>();
			
			frozenModifiedDirs = modifiedDirs;
			modifiedDirs = new HashSet<String>();
			
			frozenModifiedTrees = modifiedTrees;
			modifiedTrees = new HashSet<String>();
			
			frozenDeletedResources = deletedResources;
			deletedResources = new HashSet<String>();
		}
		
		// Doing the actual update.
		try {
			SVNURL repositoryURL = SVNURL.parseURIEncoded(url);
			
			SVNUpdateClient updateClient = ourClientManager.getUpdateClient();
	        updateClient.setIgnoreExternals(false);
	        try {
	        	/*
    			 * Trying to do an update - if it does not succeed, then most
    			 * probably the wc hasn't been checked out yet. That's why
    			 * we try to do a check out in case of an error.
    			 */
	        	lastRevision = updateClient.doUpdate(wc, SVNRevision.HEAD, 
	        			true);
	        } catch (SVNException e) {
	        	if (tryCheckout) { 
	        		updateClient.doCheckout(repositoryURL, wc, 
	        			SVNRevision.HEAD, SVNRevision.HEAD, true);
	        	} else {
	        		throw e;
	        	}
	        }
		} catch (SVNException e) {
			if (firstTime) {
				tryCleanup();
				update(tryCheckout, false);
			} else {
				log.error("Error while updating the repository.", e);
			}
		}
		
		// Clearing the just updated files.
		synchronized (synchronizer) {
			frozenDeletedResources.clear();
			frozenModifiedFiles.clear();
			frozenModifiedDirs.clear();
			frozenModifiedTrees.clear();
		}
	}
	
	private void deleteFromSet(Set<String> resources, String deletedPath) {
		for (Iterator<String> iter = resources.iterator(); iter.hasNext();) {
			if (iter.next().startsWith(deletedPath)) {
				iter.remove();
			}
		}
	}
	
	public void addToDeleted(String fullPath) {
		String suffFullPath = Tools.concatenatePaths(fullPath, "");
		
		synchronized (synchronizer) {
			/*
			 * From the modified sets, we have to remove every path that is a child
			 * of the deleted one.
			 */
			deleteFromSet(modifiedFiles, suffFullPath);
			deleteFromSet(modifiedDirs, suffFullPath);
			deleteFromSet(modifiedTrees, suffFullPath);
			deleteFromSet(frozenModifiedFiles, suffFullPath);
			deleteFromSet(frozenModifiedDirs, suffFullPath);
			deleteFromSet(frozenModifiedTrees, suffFullPath);
			
			deletedResources.add(suffFullPath);
		}
	}
	
	private boolean checkPrefixes(Set<String> resources, String path) {
		for (String prefix : resources) {
			if (path.startsWith(prefix)) {
				return true;
			}
		}
		
		return false;
	}

	public boolean isDeleted(String fullPath) {
		/*
		 * Checking if a path in the delete resources set is a prefix of the
		 * given one. If so, it means that a directory was deleted containing
		 * the given path, so the path itself is deleted too. There is one
		 * edge case, when there is a path in the set that is a prefix of the
		 * given path but is not a directory - in that case the check could
		 * return incorrect results. That's why every path is suffixed with a 
		 * /.
		 */
		
		String suffFullPath = Tools.concatenatePaths(fullPath, "");
		synchronized (synchronizer) {
			if (checkPrefixes(deletedResources, suffFullPath) ||
					checkPrefixes(frozenDeletedResources, suffFullPath)) {
				return true;
			}
		}
		
		return false;
	}
	
	public boolean isModified(String fullPath) {
		return isNodeModified(fullPath, false) || 
			isDirectoryModified(fullPath, false) ||
			isTreeModified(fullPath);
	}

	public void addNodeToModfied(String fullPath) {
		String suffFullPath = Tools.concatenatePaths(fullPath, "");
		
		/*
		 * We don't delete the modified path from the "deleted" dirty-set,
		 * because it also represents information that children of that
		 * path are also deleted. So, we remember information about which
		 * path is created anew (in case it is in the delted set also)
		 * and always check in the modified set before checking the
		 * deleted set.
		 */
		
		synchronized (synchronizer) {
			modifiedFiles.add(suffFullPath);
		}
	}

	public boolean isNodeModified(String fullPath) {
		return isNodeModified(fullPath, true);
	}
	
	public boolean isNodeModified(String fullPath, boolean checkTree) {
		String suffFullPath = Tools.concatenatePaths(fullPath, "");
		
		synchronized (synchronizer) {
			return modifiedFiles.contains(suffFullPath) || 
				frozenModifiedFiles.contains(suffFullPath) ||
				(checkTree && isTreeModified(fullPath));
		}
	}

	public void addDirectoryToModfied(String fullPath) {
		String suffFullPath = Tools.concatenatePaths(fullPath, "");
		
		synchronized (synchronizer) {
			modifiedDirs.add(suffFullPath);
		}
	}

	public boolean isDirectoryModified(String fullPath) {
		return isDirectoryModified(fullPath, true);
	}
	
	public boolean isDirectoryModified(String fullPath, boolean checkTree) {
		String suffFullPath = Tools.concatenatePaths(fullPath, "");
		
		synchronized (synchronizer) {
			return modifiedDirs.contains(suffFullPath) ||
				frozenModifiedDirs.contains(suffFullPath) ||
				(checkTree && isTreeModified(fullPath));
		}
	}
	
	public void addTreeToModfied(String fullPath) {
		String suffFullPath = Tools.concatenatePaths(fullPath, "");
		
		synchronized (synchronizer) {
			modifiedTrees.add(suffFullPath);
		}
	}

	public boolean isTreeModified(String fullPath) {
		String suffFullPath = Tools.concatenatePaths(fullPath, "");
		
		synchronized (synchronizer) {
			if (checkPrefixes(modifiedTrees, suffFullPath) ||
					checkPrefixes(frozenModifiedTrees, suffFullPath)) {
				return true;
			}
		}
		
		return false;
	}
}
