/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.shotoku.svn.service;

import java.util.Collections;
import java.util.HashMap;

import java.util.Map;

import javax.ejb.Local;

import org.apache.commons.configuration.Configuration;
import org.apache.log4j.Logger;
import org.jboss.annotation.ejb.Management;
import org.jboss.annotation.ejb.Service;
import org.jboss.shotoku.ContentManager;
import org.jboss.shotoku.svn.SvnService;
import org.jboss.shotoku.svn.SvnTools;
import org.tmatesoft.svn.core.internal.io.dav.DAVRepositoryFactory;
import org.tmatesoft.svn.core.internal.io.svn.SVNRepositoryFactoryImpl;

/**
 * @author Adam Warski (adamw@aster.pl)
 * @author Damon Sicore (damon@sicore.com)
 */
@Service(objectName=SvnService.SVN_SERVICE_NAME)
@Local(SvnServiceLocal.class)
@Management(SvnService.class)
public class SvnServiceImpl implements SvnService {
	private static final Logger log = Logger.getLogger(SvnServiceImpl.class);
	
	/**
	 * <code>DEFAULT_TIMER_INTERVAL</code> - default interval length of the 
	 * timer.
	 */
	private final static long DEFAULT_TIMER_INTERVAL = 1000*5; // 5 seconds
	private volatile long timerInterval;
	
	private Map<String, SvnRepository> repositories;
	
	/*
	 * Service lifecycle management.
	 */
	
	public void create() throws Exception {
		// Set up connection protocols support:
        // for DAV (over http and https)
        DAVRepositoryFactory.setup();

        // for SVN (over svn and svn+ssh)
        SVNRepositoryFactoryImpl.setup();
		
		repositories = Collections.synchronizedMap(
				new HashMap<String, SvnRepository>());
		
		/* 
		 * Setting up content managers - svn repositories should be added
		 * during this operation.
		 */
		ContentManager.setup();
	}
	
	public void start() throws Exception {
		// Setting the default timer interval.
		timerInterval = DEFAULT_TIMER_INTERVAL;
		
		new Thread() {
			{
				setDaemon(true);
			}
			
			public void run() {
				while (true) {
					try {
						sleep(getTimerInterval());
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					
					update();
				}
			}
		}.start();
	}

	public void stop() {
	}

	public void destroy() {
	}
	
	/*
	 * Timer-handling functions.
	 */
	
	public void setTimerInterval(long timerInterval) {
		this.timerInterval = timerInterval;
	}
	
	public long getTimerInterval() {
		return timerInterval;
	}
	
	public void update() {
		for (SvnRepository repo : repositories.values()) {
			repo.update();
		}
	}
	
	/*
	 * SvnService implementation.
	 */
	
	public void registerRepository(String id, Configuration conf) {
		if (repositories.get(id) == null) {
			SvnRepository repo = new SvnRepository(
					conf.getString(SvnTools.PROPERTY_USERNAME),
					conf.getString(SvnTools.PROPERTY_PASSWORD),
					conf.getString(SvnTools.PROPERTY_URL),
					conf.getString(SvnTools.PROPERTY_LOCALPATH));
			
			repositories.put(id, repo);
			
			// First update/ checkout.
			repo.firstUpdate();
			
			log.info("Added svn repository: " + id);
		}
	}

	public boolean isModified(String id, String fullPath) {
		return repositories.get(id).isModified(fullPath);
	}

	public void addToDeleted(String id, String fullPath) {
		repositories.get(id).addToDeleted(fullPath);
	}

	public boolean isDeleted(String id, String fullPath) {
		return repositories.get(id).isDeleted(fullPath);
	}

	public void addNodeToModfied(String id, String fullPath) {
		repositories.get(id).addNodeToModfied(fullPath);
	}

	public boolean isNodeModified(String id, String fullPath) {
		return repositories.get(id).isNodeModified(fullPath);
	}
	
	public void addTreeToModfied(String id, String fullPath) {
		repositories.get(id).addTreeToModfied(fullPath);
	}

	public boolean isTreeModified(String id, String fullPath) {
		return repositories.get(id).isTreeModified(fullPath);
	}

	public void addDirectoryToModfied(String id, String fullPath) {
		repositories.get(id).addDirectoryToModfied(fullPath);
	}

	public boolean isDirectoryModified(String id, String fullPath) {
		return repositories.get(id).isDirectoryModified(fullPath);
	}
}
